/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package compid.data.db_derby;

import compid.data.*;
import compid.data.tuples.*;
import java.util.HashMap;

/**
 *
 * @author Lari Natri
 */
public class PeptideHitDAODerby extends BaseDAODerby implements PeptideHitDAO {
    
    private final DAOFactory daoFactory;
    
    public PeptideHitDAODerby(DAOFactory daoFactory) {
        this.daoFactory = daoFactory;
    }    
    
    @Override
    public boolean setForeignKeysFromRefs(PeptideHit pephit, boolean addIfNeeded)
        throws DAOException
    {
        if (pephit.getForeignKeys()==null) {
            if (pephit.getPeptide().getPrimaryKey()<0) {
                if (addIfNeeded)
                    this.daoFactory.getPeptideDAO().add(pephit.getPeptide());
                else return false;
            }
            if (pephit.getQuery().getPrimaryKey()<0) {
                if (addIfNeeded)
                    this.daoFactory.getQueryDAO().add(pephit.getQuery());
                else return false;
            }
            HashMap<String,Object> fks = new HashMap<String,Object>(2,1);
            fks.put(pephit.getPeptide().getTableLabel()+Tuple.PKLABELPOSTFIX,pephit.getPeptide().getPrimaryKey());
            fks.put(pephit.getQuery().getTableLabel()+Tuple.PKLABELPOSTFIX,pephit.getQuery().getPrimaryKey());
            pephit.setAllForeignKeys(fks);
        }
        return true;        
    }

    @Override
    public void add(PeptideHit pephit) throws DAOException {
        setForeignKeysFromRefs(pephit, true);
        if (pephit.getPrimaryKey()>-1) {
            System.out.println("DEBUG: addPeptide: pept has been added before, no addition");
            return;
        }
        if (pephit.getPrimaryKey()==Tuple.PK_ERRORINSERTING) {
            return;
        }
        int pk = insertIfNotExists(pephit);
        pephit.setPrimaryKey(pk);
    }
    

}


