/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package compid.data.db_derby;

import compid.data.*;
import compid.data.tuples.*;
import java.util.HashMap;


/**
 *
 * @author Lari Natri
 */
public class ProtHitPeptHitJoinDAODerby extends BaseDAODerby implements ProtHitPeptHitJoinDAO {

    private final DAOFactory daoFactory;
            
    
    public ProtHitPeptHitJoinDAODerby(DAOFactory daoFactory) {
        this.daoFactory = daoFactory;
    }
    
    @Override
    public boolean setForeignKeysFromRefs(ProtHitPeptHitJoin phphj, boolean addIfNeeded)
             throws DAOException
    {

        if (phphj.getForeignKeys()==null) {
            if (phphj.getProteinHit().getPrimaryKey()<0) {
                if (addIfNeeded)
                    this.daoFactory.getProteinHitDAO().add(phphj.getProteinHit());
                else return false;
            }
            if (phphj.getPeptideHit().getPrimaryKey()<0) {
                if (addIfNeeded)
                    this.daoFactory.getPeptideHitDAO().add(phphj.getPeptideHit());
                else return false;
            }
            HashMap<String,Object> fks = new HashMap<String,Object>(2,1);
            fks.put(phphj.getProteinHit().getTableLabel()+Tuple.PKLABELPOSTFIX,phphj.getProteinHit().getPrimaryKey());
            fks.put(phphj.getPeptideHit().getTableLabel()+Tuple.PKLABELPOSTFIX,phphj.getPeptideHit().getPrimaryKey());
            phphj.setAllForeignKeys(fks);
        }
        return true;
        
    }

    @Override
    public void add(ProtHitPeptHitJoin phphj)
            throws DAOException
    {
        this.setForeignKeysFromRefs(phphj, true);
        if (phphj.getPrimaryKey()>-1) {
            System.out.println("DEBUG: addPeptide: pept has been added before, no addition");
            return;
        }
        if (phphj.getPrimaryKey()==Tuple.PK_ERRORINSERTING) {
            return;
        }
        int pk = insertIfNotExists(phphj);
        phphj.setPrimaryKey(pk);
    }
    


}
