/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package compid.data.db_derby;

import compid.data.*;
import compid.data.tuples.*;
import java.util.HashMap;

/**
 *
 * @author Lari Natri
 */
public class ProtSeqDAODerby extends BaseDAODerby implements ProtSeqDAO {
    
    private final DAOFactory daoFactory;
    
    public ProtSeqDAODerby(DAOFactory daoFactory) {
        this.daoFactory = daoFactory;
    }    

    @Override
    public void add(ProtSeq protSeq) throws DAOException {
        // this tuple has no foreign keys, so make it empty
        if (protSeq.getForeignKeys()==null)
            protSeq.setAllForeignKeys(new HashMap<String,Object>(0,1));
        if (protSeq.getPrimaryKey()>-1) {
            System.out.println("DEBUG: addProtSeq: protein sequence has " +
                    "been added before, no addition");
            return;
        }
        if (protSeq.getPrimaryKey()==Tuple.PK_ERRORINSERTING) {
            return;
        }
        int pk = insertIfNotExists(protSeq);
        protSeq.setPrimaryKey(pk);
    }

}
