/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package compid.data.db_derby;

import java.util.HashMap;

import compid.data.*;
import compid.data.tuples.*;


/**
 *
 * @author Lari Natri
 */
public class ProteinDAODerby extends BaseDAODerby implements ProteinDAO {

    private final DAOFactory daoFactory;
    
    public ProteinDAODerby(DAOFactory daoFactory) {
        this.daoFactory = daoFactory;
    }    

    @Override
    public void add(Protein prot) throws DAOException {

        setForeignKeysFromRefs(prot, true);
        if (prot.getPrimaryKey()>-1) {
            System.out.println("DEBUG: addProtein: protein has been added before, no addition");
            return;
        }
        if (prot.getPrimaryKey()==Tuple.PK_ERRORINSERTING) {
            return;
        }
        
        // If this protein has a sequence which is already in the database,
        // other proteins with the same accession as this will be updated
        // to point to the sequence in this prot.
        if (prot.getProtSeq().getPrimaryKey()>0 && prot.getProtSeq()
                .getLength()>0 && prot.getAccession().length()>0)
        {
            this.daoFactory.getUpdateHandler().updateProteinsSeqId(
                    prot.getAccession(), prot.getProtSeq().getPrimaryKey());
        }

        int pk = insertIfNotExists(prot);

        prot.setPrimaryKey(pk);
    }

    
    @Override
    public boolean setForeignKeysFromRefs(Protein prot, boolean addIfNeeded)
             throws DAOException
    { 
        if (prot.getProtDbInfo()==null || prot.getProtSeq()==null) {
            return true;
        }
        if (prot.getForeignKeys()==null) {
            if (prot.getProtDbInfo().getPrimaryKey()<0) {
                if (addIfNeeded)
                    this.daoFactory.getProtDbInfoDAO().add(prot.getProtDbInfo());
                else return false;
            }
            if (prot.getProtSeq().getPrimaryKey()<0) {
                if (addIfNeeded)
                    this.daoFactory.getProtSeqDAO().add(prot.getProtSeq());
                else return false;
            }

            HashMap<String,Object> fks = new HashMap<String,Object>(2,1);
            fks.put(prot.getProtDbInfo().getTableLabel()+Tuple.PKLABELPOSTFIX,prot.getProtDbInfo().getPrimaryKey());
            fks.put(prot.getProtSeq().getTableLabel()+Tuple.PKLABELPOSTFIX,prot.getProtSeq().getPrimaryKey());
            prot.setAllForeignKeys(fks);
        }
        return true;
    }

    
}
