/*
 * Copyright 2009, 2010 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package compid.data.db_derby;

import compid.data.*;
import compid.data.tuples.*;
import java.util.HashMap;

/**
 *
 * @author Lari Natri
 */
public class ProteinHitDAODerby extends BaseDAODerby implements ProteinHitDAO {
    
    private final DAOFactory daoFactory;
    
    
    public ProteinHitDAODerby(DAOFactory daoFactory) {
        this.daoFactory = daoFactory;
    }
    
    @Override
    public boolean setForeignKeysFromRefs(ProteinHit prothit, boolean addIfNeeded)
             throws DAOException
    {
        if (prothit.getForeignKeys()==null) {
            if (prothit.getProtein().getPrimaryKey()<0) {
                if (addIfNeeded)
                    this.daoFactory.getProteinDAO().add(prothit.getProtein());
                else return false;
            }
            if (prothit.getDataset().getPrimaryKey()<0) {
                if (addIfNeeded)
                    this.daoFactory.getDataSetInfoDAO().add(prothit.getDataset());
                else return false;
            }
            HashMap<String,Object> fks = new HashMap<String,Object>(2,1);
            fks.put(prothit.getProtein().getTableLabel()+Tuple.PKLABELPOSTFIX,prothit.getProtein().getPrimaryKey());
            fks.put(prothit.getDataset().getTableLabel()+Tuple.PKLABELPOSTFIX,prothit.getDataset().getPrimaryKey());
            prothit.setAllForeignKeys(fks);
        }
        return true;
    }

    @Override
    public void add(ProteinHit prothit) throws DAOException {
        setForeignKeysFromRefs(prothit, true);
        if (prothit.getPrimaryKey()>-1) {
            System.out.println("DEBUG: addProteinHit: proteinhit has been added before, no addition");
            return;
        }
        if (prothit.getPrimaryKey()==Tuple.PK_ERRORINSERTING) {
            return;
        }
        int pk = insertIfNotExists(prothit);
        prothit.setPrimaryKey(pk);
    }
    

}
