/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package compid.data.db_derby;

import compid.data.*;
import compid.data.tuples.*;
import java.util.HashMap;

/**
 *
 * @author Lari Natri
 */
public class QueryDAODerby extends BaseDAODerby implements QueryDAO {
    
    private final DAOFactory daoFactory;
    
    public QueryDAODerby(DAOFactory daoFactory) {
        this.daoFactory = daoFactory;
    }    
    
    @Override
    public boolean setForeignKeysFromRefs(Query query, boolean addIfNeeded)
             throws DAOException
    {
        if (query.getForeignKeys()==null) {
            if (query.getDataSetInfo().getPrimaryKey()<0) {
                if (addIfNeeded)
                    this.daoFactory.getDataSetInfoDAO().add(query.getDataSetInfo());
                else return false;
            }
            HashMap<String,Object> fks = new HashMap<String,Object>(1,1);
            fks.put(query.getDataSetInfo().getTableLabel()+Tuple.PKLABELPOSTFIX,(Integer)query.getDataSetInfo().getPrimaryKey());
            query.setAllForeignKeys(fks);
        }
        return true;
        
    }

    @Override
    public void add(Query query) throws DAOException {
        this.setForeignKeysFromRefs(query, true);
        if (query.getPrimaryKey()>-1) {
            System.out.println("DEBUG: addQuery: Query has been added before, no addition");
            return;
        }
        if (query.getPrimaryKey()==Tuple.PK_ERRORINSERTING) {

            return;
        }
        int pk = insertIfNotExists(query);
        query.setPrimaryKey(pk);
    }


}
