/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package compid.report;

import compid.data.tuples.Tuple;
import java.util.Collection;
import java.util.HashSet;

/**
 *
 * @author Lari Natri
 */
public class TupleCollection<T extends Tuple> {

    private final Collection<T> collection;
    private final String name;

    public TupleCollection(Collection<T> coll, String name) {
        if (coll==null) {
            this.collection = new HashSet<T>(); // was list
        } else {
            this.collection = coll;
        }
        this.name = name;
    }

    public TupleCollection(String name) {
        this.collection = new HashSet<T>(); // was list
        this.name = name;
    }

    public Collection<T> getCollection() {
        return this.collection;
    }

    public final int size() {
        synchronized (this.collection) {
            return this.collection.size();
        }
    }

    public final String getName() {
        return this.name;
    }

    public final void add(T t) {
        synchronized (this.collection) {
            this.collection.add(t);
        }
    }
}
