/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package compid.sequences;

/**
 * Abstract base class for components which can be part of a {@link Sequence}.
 * @author Lari Natri
 */
public abstract class Component
{

    protected Component()
    {
    }

    public abstract boolean isCompatible(Component other);

    @Override
    public abstract boolean equals(Object other);
    
    public abstract boolean equalsLoosely(Component other);


    @Override
    public abstract int hashCode();

    public String getName()
    {
        return "unknown";
    }

    public static boolean areCompatible(Component a, Component b)
    {
        if (a.isCompatible(b) && b.isCompatible(a)) {
            return true;
        } else {
            return false;
        }
    }

    @Override
    public String toString()
    {
        return this.getName();
    }
}
