/*
 * Copyright 2009 Dem Pilafian, Lari Natri
 *
 * No license, public domain software
 */
package net.aavalla.utils.general;

import java.lang.reflect.Method;
import java.net.URL;
import java.util.Arrays;

/**
 * <b>Bare Bones Browser Launch for Java</b><br>
 * Utility class to open a web page from a Swing application
 * in the user's default browser.<br>
 * Supports: Mac OS X, GNU/Linux, Unix, Windows XP/Vista<br>
 * Example Usage:<code><br> &nbsp; &nbsp;
 *    String url = "http://www.google.com/";<br> &nbsp; &nbsp;
 *    BareBonesBrowserLaunch.openURL(url);<br></code>
 * Latest Version: <a href="http://www.centerkey.com/java/browser/">www.centerkey.com/java/browser</a><br>
 * Author: Dem Pilafian<br>
 * Public Domain Software -- Free to Use as You Like
 * @version 2.0, May 26, 2009 + edits by Lari Natri
 */
public class BareBonesBrowserLaunch {

    /**
     * Browser executables to try (in order) on UNIX-like platforms
     */
    public static final String[] browsers = {"xdg-open", "firefox", "opera",
        "konqueror", "chromium-browser", "rekonq", "arora", "swiftfox",
        "seamonkey", "epiphany", "galeon", "kazehakase", "mozilla", "netscape",
        "abrowser", "chromium"}; // NOI18N

    /**
     * Opens the specified web page in a web browser
     * @param url A web address (URL) of a web page")
     * @throws RuntimeException
     */
    public static void openURL(URL url) throws RuntimeException {
        openURL(url.toString());
    }

    /**
     * Opens the specified web page in a web browser
     * @param url A web address (URL) of a web page")
     * @throws RuntimeException
     */
    public static void openURL(String url) throws RuntimeException {
        String osName = System.getProperty("os.name"); // NOI18N
        try {
            if (osName.startsWith("Mac OS")) { // NOI18N
                Class<?> fileMgr = Class.forName("com.apple.eio.FileManager"); // NOI18N
                Method openURL = fileMgr.getDeclaredMethod("openURL",
                        new Class[]{String.class}); // NOI18N
                openURL.invoke(null, new Object[]{url});

            } else if (osName.startsWith("Windows")) { // NOI18N
                Runtime.getRuntime().exec("rundll32 url.dll,FileProtocolHandler " +
                        url); // NOI18N
            } else { // assume Unix or Linux
                boolean found = false;
                for (String browser : browsers) {
                    if (!found) {
                        found = Runtime.getRuntime().exec(
                                new String[]{"which", browser}).waitFor() == 0; // NOI18N
                        if (found) {
                            Runtime.getRuntime().exec(new String[]{browser, url});
                        }
                    }
                }
                if (!found) {
                    throw new RuntimeException(Arrays.toString(browsers));
                }
            }
        } catch (Exception e) {
            throw new RuntimeException("Error when launching web browser: \n" +
                    e.toString());
        }
    }
}
