/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.aavalla.utils.general;

/**
 * Some utility methods for parsing FASTA files
 * @author Lari Natri
 */
public class FastaUtils {

    /**
     * Type of a line in a FASTA file
     */
    public static enum FastaLineType { COMMENT, HEADER, SEQUENCE, OTHER };

    /**
     * Returns the type of the given line
     * @param fastaLine one line in a FASTA file
     * @return type of the line
     */
    public static FastaLineType getFastaLineType(final String fastaLine) {
        if (fastaLine == null) return FastaLineType.OTHER;
        if (fastaLine.length() < 1) return FastaLineType.OTHER;
        if (fastaLine.startsWith(";")) return FastaLineType.COMMENT; //NOI18N
        if (fastaLine.startsWith("#")) return FastaLineType.COMMENT; //NOI18N
        if (fastaLine.startsWith(">")) return FastaLineType.HEADER; //NOI18N
        return FastaLineType.SEQUENCE;
    }
}
