/*
 *  Copyright 2010 Lari Natri <lari.natri@gmail.com>.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.aavalla.utils.gui.twovenn;

import net.aavalla.utils.jobpool.parameters.FieldAnnotations;
import net.aavalla.utils.jobpool.parameters.OutputFile;
import net.aavalla.utils.jobpool.parameters.ParameterCollection;
import net.aavalla.utils.jobpool.parameters.validators.OutputFileValidator;

/**
 *
 * @author Lari Natri <lari.natri@gmail.com>
 */
public class VennExportParameters extends ParameterCollection {

    public enum ImageType {
        PNG,
        JPG
    }
    
    private VennParameters p = null;

    @FieldAnnotations(name="Image format",
        description="<b>PNG</b>: lossless with transparent background, <b><i>recommended</i></b>\n"
                    + "<b>JPG</b>: worse quality, colored background")
    private ImageType type = ImageType.PNG;
    @FieldAnnotations(name="Output file",
        description="Image file to write into",
        validator=OutputFileValidator.class)
    private OutputFile file = null;
    @FieldAnnotations(name="Width",
        description="Image width (x) in pixels")
    private int x = 500;
    @FieldAnnotations(name="Height",
        description="Image height (y) in pixels")
    private int y = 500;

    public VennExportParameters() {
        this.setDescription("Exports Venn diagram to an image file");
    }

    @Override
    public synchronized String validateFieldRelations() {
        String fname = this.file.getName().toLowerCase();
        if (this.type.equals(ImageType.JPG) && !fname.endsWith(".jpg") && !fname.endsWith(".jpeg")) {
            return "File name extension must be .jpg, when using JPG";
        }
        if (this.type.equals(ImageType.PNG) && !fname.endsWith(".png")) {
            return "File name extension must be .png, if using PNG";
        }
        return null;
    }

    synchronized void setParams(VennParameters p, int x, int y) {
        this.p = p;
        this.x = x;
        this.y = y;
    }

    public OutputFile getFile() {
        return this.file;
    }

    public ImageType getType() {
        return this.type;
    }

    public synchronized int getX() {
        return this.x;
    }

    public synchronized int getY() {
        return this.y;
    }

    public synchronized VennParameters getVennParameters() {
        return this.p;
    }


}
