/*
 * Copyright 2010 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool;

/**
 *
 * @author Lari Natri
 */
public final class JobResultEvent<R> {

    private final R result;
    private final Job job;
    private final Throwable failReason;
    private final boolean cancelled;

    JobResultEvent(R result, Throwable failReason, boolean cancelled, Job job) {
        this.result = result;
        this.failReason = failReason;
        this.cancelled = cancelled;
        this.job = job;
    }

    public final R getResult() {
        return result;
    }

    public final Throwable getFailReason() {
        return this.failReason;
    }

    public final boolean failed() {
        return this.failReason!=null;
    }

    public final boolean cancelled() {
        return this.cancelled;
    }

    public final Job getSource() {
        return this.job;
    }

}
