/*
 * Copyright 2010 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool;

/**
 * Represents a job's current state.
 * @author Lari Natri
 */
public enum JobState {
    CREATED { // constructed, not yet registered to a JobPool; set by Job constructor
        @Override public String toString() { return "Created"; }
    },
    READY { // in queue, ready to run, registered to a JobPool; set by JobPool
        @Override public String toString() { return "Queued"; }
    },
    WAITING { // waiting for a subjob to complete; set by JobPool
        @Override public String toString() { return "Waiting"; }
    },
    RUNNING { // running; set by JobPool
        @Override public String toString() { return "Running"; }
    },
    FINISHED { // finished succesfully; set by Job.run()
        @Override public String toString() { return "Finished"; }
    },
    FAILED { // failed (threw an exception); set by Job.run()
        @Override public String toString() { return "Failed"; }
    },
    CANCELLED { // cancelled; set by Job.run()
        @Override public String toString() { return "Cancelled"; }
    }
}

