/*
 * Copyright 2010 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;

/**
 *
 * @author Lari Natri
 */
public class Tester {

    public static final Object testLock = new Object();
    public static int counter = 0;

    public class TestJob2 extends Job<String,String> {

        public TestJob2(String p) {
            super(p);
            this.setSupportsCancellation(true);
            this.setName("TestJob2 with param="+p);
        }

        @Override protected String work(String p) {
            int myCount;
            synchronized(testLock) {
                myCount = counter++;
            }
            this.reportProgress(50);
            if (myCount<20) {
                try {
                    this.addSubjob(new TestJob2(p+"nowait"));
                    this.addSubjob(new TestJob1(p+"nowait2"));
                    //this.waitSubJob(new TestJob2(p+"jee"));
                    Thread.sleep(1000);
                } catch (Throwable ex) {
                    System.out.println("failed "+ex);
                }
            }
            
            return (p+"->res2");
        }

    }

    public class TestJob1 extends Job<String,String> {

        public TestJob1(String p) {
            super(p);
            this.setSupportsCancellation(true);
            this.setName("TestJob1 with param="+p);
            this.addCategory(JobCategory.CPU);
        }


        @Override protected String work(String p) throws Throwable {
            TestJob2 sub2 = new TestJob2(p+"pTJ1");
            TestJob2 sub1 = new TestJob2(p+"pTJ2");
            String res = null;
            this.addSubjob(sub2);
            for (int i=0; i<10; ++i) {
                if (cancelPending()) throw CANCELLEDRESULT;
                try { Thread.sleep(100); } catch (Exception ex) {}
            }

//                res = this.waitSubJob(sub1);
//                System.out.println("SUBJOB RESULT: "+res+" for parent job "+p);
//                try { Thread.sleep(800); } catch (Exception ex) {}
//                this.waitSubJob(sub2);
            //System.out.println("SUBJOB RESULT: "+res+" for parent job "+p);
                
            return p+"->res1";
        }

    }

    public void test() {
        JobPool pool = new JobPool();
        pool.setMaxConcurrencyForCategory(JobCategory.CPU, 1);

        JFrame f = new JFrame();
        JScrollPane s = new JScrollPane();
        JPanel p = new JPanel();
        s.setViewportView(p);
        
        p.add(pool.getNewJobTreeForm(true));
//        p.add(new JobPoolForm(pool,true));
//        p.add(new JobPoolForm(pool,true));
//        p.add(new JobPoolForm(pool,true));
//        p.add(new JobPoolForm(pool,true));
        f.add(s);
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        f.setSize(600,500);
        f.setVisible(true);


        Job testJobA = new TestJob1("pA");
        Job testJobB = new TestJob1("pB");
        pool.addJob(testJobA);
        pool.addJob(testJobB);

    }


    private static long calc(long n, int p, long a) {
        int p2 = p*p;
        long a2 = a*a;
        long n2 = n*n;
        double res = (2*p*n-3*p - Math.sqrt( 4*p2*n2 - 4*p2*n + p2 + 4*p2*a + 4*p2*a2 - 8*p2*a*n - 4*p*n2 + 4*p*n )) / (2*p);

        return Math.round(Math.ceil(res));

    }




    public static void main(String[] args) {

//
//        for (long n=2; n<30000; ++n) {
//            for (int p=1; p<=n; ++p) {
//
//                long a = 0;
//                long x = x = calc(n,p,a);
//
//                if (x<=0) {
//                    x = 1;
//                }
//
//                long opsAll = 0;
//
//                for (int c=0; c<p; ++c) {
//
////                    System.out.println("Partition "+c+": a="+a+", x="+x);
//                    long opsInP = 0;
//                    for (long i=a; i<=x; ++i) {
//                        assert(a<=x);
//                        for (long j=i+1; j<n; ++j) {
//                            assert(i<j);
//                            ++opsAll;
//                            ++opsInP;
//                        }
//
//                    }
////                    System.out.println("Operations in partition "+c+": "+opsInP);
//
//                    a = x+1;
//                    if (a>=n) { break;}
//                    x = calc(n,p,a);
//                    if (x<=0) {
//                        System.out.println("ASDÖLJKASDLJSDA n="+n+", p="+p+", x="+x);
//                        x = n-1;
//                    }
//
//                    if (c==p-1 && x<n-1) ++p; // must have this unless use ceiling always
//
//                }
//                assert(p>0 && p<=n);
//                System.out.println("Operations summed:         "+opsAll+", p="+p+", n="+n);
//
//                long opsShould = 0;
//                for (int i=0; i<n; ++i) {
//                    for (int j=i+1; j<n; ++j) {
//                        ++opsShould;
//                    }
//                }
////                System.out.println("Operations should have:    "+opsShould);
////                System.out.println("Operations/part. optimal:  "+Math.round(opsShould/p));
//
//                assert(opsShould==(n*n-n)/2);
//                assert(opsAll==opsShould);
//            }
//        }


        final Tester tester = new Tester();
        try {
        SwingUtilities.invokeAndWait(new Runnable() {

            public void run() {
                tester.test();
            }
        });
        } catch (Exception ex) {}
        

        System.gc();
        System.gc();
        System.gc();
        try {
            Thread.sleep(10000);
            System.out.println("Main exiting");
        } catch (Exception ex) {}
        System.gc();
        System.gc();
        System.gc();
        System.exit(0);

    }

}
