/*
 * Copyright 2010 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool.gui;

import net.aavalla.utils.jobpool.Job;
import net.aavalla.utils.jobpool.ParameterizedJob;
import net.aavalla.utils.jobpool.parameters.ParameterCollection;
import net.aavalla.utils.jobpool.parameters.gui.ParameterDialog;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.AbstractAction;
import javax.swing.JLabel;
import javax.swing.JPopupMenu;
import javax.swing.ToolTipManager;
import javax.swing.UIManager;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

/**
 *
 * @author Lari Natri
 */
public class JobPoolForm extends javax.swing.JPanel  {

    private final JobTreeRepainter repainter;

    /** Creates new form JobPoolForm */
    public JobPoolForm(final TreeModel model, DefaultMutableTreeNode runningNode, final boolean popup) {
        initComponents();
        
        this.jobPoolTree.setModel(model);
        this.jobPoolTree.setCellRenderer(new JobTreeCellRenderer());
        this.jobPoolTree.setEditable(false);
        this.jobPoolTree.setRootVisible(false);
        this.jobPoolTree.setShowsRootHandles(true);
        this.jobPoolTree.addMouseListener(new ClickListener());
        this.jobPoolTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        ToolTipManager.sharedInstance().registerComponent(this.jobPoolTree);

        this.repainter = new JobTreeRepainter(jobPoolTree,runningNode,
                UIManager.getInt("ProgressBar.repaintInterval"),
                ((TreeNode)model.getRoot()).getChildCount()+1);

    }

    @Override
    protected void finalize() throws Throwable {
        repainter.dispose();
        super.finalize();
    }

    @Override
    public void setVisible(boolean b) {
        super.setVisible(b);
        if (b) {
            this.repainter.maybeRestart();
        } else {
            this.repainter.stop();
        }
    }



    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jobTreeScrollPane = new javax.swing.JScrollPane();
        jobPoolTree = new javax.swing.JTree();

        javax.swing.tree.DefaultMutableTreeNode treeNode1 = new javax.swing.tree.DefaultMutableTreeNode("root");
        jobPoolTree.setModel(new javax.swing.tree.DefaultTreeModel(treeNode1));
        jobPoolTree.setToolTipText("");
        jobPoolTree.setMaximumSize(new java.awt.Dimension(32767, 32767));
        jobPoolTree.setRootVisible(false);
        jobPoolTree.setShowsRootHandles(true);
        jobTreeScrollPane.setViewportView(jobPoolTree);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jobTreeScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 421, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jobTreeScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 233, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTree jobPoolTree;
    private javax.swing.JScrollPane jobTreeScrollPane;
    // End of variables declaration//GEN-END:variables


    
    private final class JobControlMenu extends JPopupMenu {
        private final class CancelAction extends AbstractAction {
            public CancelAction(String title) { super(title); }
            @Override public void actionPerformed(ActionEvent e) {
                targetJob.setCancelPending(true);
            }
        }

        private final class ShowParametersAction extends AbstractAction {
            public ShowParametersAction(String title) { super(title); }
            @Override public void actionPerformed(ActionEvent e) {
                if (targetJob instanceof ParameterizedJob) {
                    ParameterCollection params = ((ParameterizedJob)targetJob).getFinalParametersForDisplay();
                    if (params==null) {
                        return;
                    }
                    ParameterDialog dialog = new ParameterDialog(null, false, params, null);
                    dialog.setVisible(true);
                }
            }
        }

        private final Job targetJob;

        private JobControlMenu(Job j) {
            super();
            JLabel title = new JLabel("<html><b><u>"+j.getName()+"</u></b></html>");
            targetJob = j;
            title.setEnabled(false);
            this.add(title);

            if (targetJob instanceof ParameterizedJob) {
                ParameterCollection params = ((ParameterizedJob)targetJob).getFinalParametersForDisplay();
                if (params!=null) {
                    ShowParametersAction paramsAction = new ShowParametersAction("Display parameters");
                    paramsAction.setEnabled(true);
                    this.add(paramsAction);
                }

            }

            CancelAction cancelAction = new CancelAction("Cancel Job");
            cancelAction.setEnabled(j.supportsCancellation() && !j.cancelPending());
            this.add(cancelAction);
            
        }
    }

    private class ClickListener extends MouseAdapter {
        @Override public void mousePressed(MouseEvent e) { maybeShowPopup(e); }
        @Override public void mouseReleased(MouseEvent e) { maybeShowPopup(e); }

        private void maybeShowPopup(MouseEvent e) {
            if (e.isPopupTrigger()) {
                TreePath path = jobPoolTree.getPathForLocation(e.getX(), e.getY());
                if (path==null) return;
                Object o = path.getLastPathComponent();
                if (o==null) return;
                if (o instanceof DefaultMutableTreeNode) {
                    Object value = ((DefaultMutableTreeNode)o).getUserObject();
                    if (value instanceof Job) {
                        Job j = (Job)value;
                        JobControlMenu menu = new JobControlMenu(j);
                        menu.show(e.getComponent(),e.getX(),e.getY());
                    } else {

                    }
                }
            }
        }
    } // ClickListener

}
