/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool.parameters;

import net.aavalla.utils.jobpool.parameters.gui.*;
import java.awt.Color;
import java.awt.Font;
import java.io.File;


/**
 *
 * @author Lari Natri
 */
public class DefaultFieldTypeHandler implements FieldTypeHandler {

    @Override
    public Class getAsSupportedType(final Class rawType) {
        Class testType = rawType;
        Class supportedType = null;
        while (supportedType==null && testType!=null && !testType.equals(Object.class)) {
            if (testType.equals(int.class) || testType.equals(Integer.class)) {
                supportedType = Integer.class;
            } else if (testType.equals(boolean.class) || testType.equals(Boolean.class)) {
                supportedType = Boolean.class;
            } else if (testType.equals(double.class) || testType.equals(Double.class)) {
                supportedType = Double.class;
            } else if (testType.equals(long.class) || testType.equals(Long.class)) {
                supportedType = Long.class;
            } else if (testType.equals(String.class)) {
                supportedType = String.class;
            } else if (testType.equals(OutputFile.class)) {
                supportedType = OutputFile.class;
            } else if (testType.equals(InputFile.class)) {
                supportedType = InputFile.class;
            } else if (testType.equals(File.class)) {
                supportedType = File.class;
            } else if (testType.equals(Color.class)) {
                supportedType = Color.class;
            } else if (testType.equals(Font.class)) {
                supportedType = Font.class;
            } else if (testType.equals(ParameterCollection.class)) {
                supportedType = ParameterCollection.class;
            } else if (testType.equals(SyncEnumSet.class)) {
                supportedType = testType;
            } else if (testType.isEnum()) {
                supportedType = testType;
            }

            if (supportedType==null) {
                testType = testType.getSuperclass();
            }
        }
        return supportedType;
    }

    @Override
    public Object convertType(final Object value, final Class valueType, final Class returnType) {

        if (returnType.equals(Integer.class)) {
            if (valueType.equals(Long.class)) { // conversion
                return new Integer(((Long)value).intValue());
            }
        } else if (returnType.equals(Double.class)) {
            if (valueType.equals(Float.class)) { // conversion
                return new Double((Float)value);
            }
            if (valueType.equals(Long.class)) { // conversion
                return new Double((Long)value);
            }
            if (valueType.equals(Integer.class)) { // conversion
                return new Double((Integer)value);
            }
        } else if (returnType.equals(Long.class)) {
            if (valueType.equals(Integer.class)) { // conversion
                return new Long((Integer)value);
            }
        } else if (returnType.equals(OutputFile.class)) {
            return new OutputFile(((File)value).toString());
        } else if (returnType.equals(InputFile.class)) {
            return new InputFile(((File)value).toString());
        }

        return value;

    }



    @Override
    public ValueComponent getValueComponentFor(Class supportedType, FieldInstance f, boolean allowEdit) {
            if (supportedType.equals(Boolean.class)) {
                return new BooleanComponent(f, allowEdit);
            } else if (supportedType.equals(Integer.class)) {
                return new IntegerComponent(f,allowEdit);
            } else if (supportedType.equals(Double.class)) {
                return new DoubleComponent(f,allowEdit);
            } else if (supportedType.equals(File.class) || supportedType.equals(OutputFile.class) || supportedType.equals(InputFile.class)) {
                return new FileChooserComponent(f, allowEdit);
            } else if (supportedType.equals(String.class)) {
                return new StringComponent(f,allowEdit);
            } else if (supportedType.equals(Color.class)) {
                return new ColorChooserComponent(f, allowEdit);
            } else if (supportedType.equals(Font.class)) {
                return new FontChooserComponent(f, allowEdit);
            } else if (supportedType.equals(SyncEnumSet.class)) {
                return new EnumSetComponent(f,allowEdit);
            } else if (supportedType.isEnum()) {
                //@SuppressWarnings("unchecked") // TODO: go checked
                return new EnumComponent(f, allowEdit);
            } else {
                return null;
            }

    }

}
