/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool.parameters;

import net.aavalla.utils.jobpool.parameters.validators.ParameterValidator;
import net.aavalla.utils.jobpool.parameters.validators.DefaultValidator;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;


import static java.lang.annotation.ElementType.FIELD;
import static java.lang.annotation.RetentionPolicy.RUNTIME;

/**
 *
 * @author Lari Natri
 */
@Retention(RUNTIME) @Target({FIELD})
public @interface FieldAnnotations {

    public static final String NULL = "$$$[NULL]$$$_represents_null"; // NOI18N

    /** Name of the parameter */
    String name() default "Unknown";
    /** Parameter description */
    String description() default "Description not available";
    /** True, if parameter is 'internal' ie doesn't require user input */
    boolean internal() default false;
    /** Set to true, if parameter is optional */
    boolean optional() default false;
    /** Validator for the field */
    Class<? extends ParameterValidator> validator() default DefaultValidator.class;
    /** Settings for the validator */
    String[] validatorSettings() default NULL;
    /** Line count, only used with String fields */
    int lines() default 1;
}
