/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool.parameters;

import net.aavalla.utils.jobpool.parameters.gui.ValueComponent;

/**
 * Handles an assortment of field types.
 * @author Lari Natri
 */
public interface FieldTypeHandler {

    /**
     * This function should convert a given type to a compatible type that is
     * supported by this FieldTypeHandler. If this handler doesn't support
     * the given type, <code>null</code> value should be returned.
     * @param rawType type to convert to a supported type (if any)
     * @return supported type or null if not supported
     */
    public Class getAsSupportedType(final Class rawType);

    /**
     * Should cast the parameter to type T. This conversion is possible,
     * because it has already passed getAsSupportedType
     * @param <T>
     * @param value
     * @return
     */
    public Object convertType(final Object value, final Class valueType, final Class returnType);

    /**
     * This function should return a new object derived from ValueComponent,
     * which can handle fields of the type given as parameter.
     * @param <T>
     * @param supportedType some type this handler supports
     * @param allowEdit whether ValueComponent should allow editing of the field
     * @return an object derived from ValueComponent
     */
    public ValueComponent getValueComponentFor(final Class supportedType, FieldInstance f, boolean allowEdit);

}
