/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool.parameters.gui;

import net.aavalla.utils.jobpool.parameters.FieldInstance;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JColorChooser;

/**
 *
 * @author Lari Natri
 */
public class ColorChooserComponent extends ValueComponent<Color> {

    /** Creates new form FileChooser */
    public ColorChooserComponent(final FieldInstance field, boolean allowEdit) {
        super(field, allowEdit);
        initComponents();

        this.colorChooserButton.setVisible(allowEdit);
        this.colorChooserButton.setToolTipText(this.getToolTipText());
        this.colorChooserButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/aavalla/utils/jobpool/parameters/gui/resources/format-stroke-color.png"))); //NOI18N
        this.colorText.setToolTipText("<html>Background is painted with the chosen color; <br/>RGB values are displayed.</html>");

        if (field.getValue()==null) {
            this.setFieldValue(Color.WHITE);
        } else {
            this.setFieldValue((Color)field.getValue());
        }

        if (allowEdit) {
            this.colorChooserButton.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    Color newColor = JColorChooser.showDialog(ColorChooserComponent.this, field.getName(), (Color)field.getValue());
                    if (newColor!=null) {
                        ColorChooserComponent.this.setFieldValue(newColor);
                    }
                }
            });
        }

    }

    @Override
    protected void updateUIWithValue(Color newValue) {
        if (newValue==null) {
            this.colorText.setText("No color selected");
            this.colorText.setForeground(this.getForeground());
            this.colorText.setBackground(this.getBackground());
        }
        this.colorText.setBackground(newValue);
        if (newValue.getRed()<128 && newValue.getBlue()<128 && newValue.getGreen()<128) {
            this.colorText.setForeground(Color.WHITE);
        } else {
            this.colorText.setForeground(Color.BLACK);
        }
        this.colorText.setText(newValue==null ? "" : "Red="+newValue.getRed()+", Green="+newValue.getGreen()+", Blue="+newValue.getBlue());
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        colorChooserButton = new javax.swing.JButton();
        colorText = new javax.swing.JTextField();

        setName("Form"); // NOI18N

        colorChooserButton.setText("Choose"); // NOI18N
        colorChooserButton.setName("colorChooserButton"); // NOI18N

        colorText.setEditable(false);
        colorText.setName("colorText"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(colorChooserButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(colorText, javax.swing.GroupLayout.DEFAULT_SIZE, 229, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                .addComponent(colorChooserButton)
                .addComponent(colorText, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton colorChooserButton;
    private javax.swing.JTextField colorText;
    // End of variables declaration//GEN-END:variables


}
