/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool.parameters.gui;

import net.aavalla.utils.jobpool.parameters.FieldInstance;
import java.awt.Color;
import java.text.ParseException;
import javax.swing.JFormattedTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

/**
 *
 * @author Lari Natri
 */
public class DoubleComponent extends ValueComponent<Double> implements DocumentListener {

    private final JFormattedTextField inputTextField;
    private int lastCursorPos = -1;
    private String lastAsText = null;

    public DoubleComponent(FieldInstance field, boolean allowEdit) {
        super(field, allowEdit);

        inputTextField = new JFormattedTextField();
        inputTextField.getDocument().addDocumentListener(this);
        this.inputTextField.setToolTipText(this.getToolTipText());

        this.setFieldValue((Double)field.getValue()); // set default value

        this.inputTextField.setBackground(Color.WHITE);
        this.inputTextField.setEditable(allowEdit);
//        this.setEnabled(allowEdit);

        this.add(this.inputTextField);

        this.validate();
    }

    @Override
    protected void updateUIWithValue(Double newValue) {
        inputTextField.getDocument().removeDocumentListener(this);
        String newStringified = newValue.toString();
        try {
            if (lastAsText!=null && Double.parseDouble(newStringified)==Double.parseDouble(lastAsText)) {
                newStringified = lastAsText;
            }
        } catch(NumberFormatException ex) {}

        inputTextField.setText(newStringified);
        if (lastCursorPos >= 0) {
            if (lastCursorPos >= newStringified.length()) {
                inputTextField.setCaretPosition(newStringified.length());
            } else {
                inputTextField.setCaretPosition(lastCursorPos);
            }
        }
        inputTextField.getDocument().addDocumentListener(this);
    }

    @Override
    public void changedUpdate(DocumentEvent e) {
        try {
            inputTextField.commitEdit();
        } catch (ParseException ex) { // shouldn't happen
            ex.printStackTrace();
        }
        String res = inputTextField.getText().trim();
        boolean isNegative = res.startsWith("-");
        res = res.replaceAll("[^0123456789.]","");
        if (isNegative) res = "-" + res;

        int firstPoint = res.indexOf(".");
        int lastPoint = res.lastIndexOf(".");
        while (firstPoint!=-1 && firstPoint!=lastPoint) {
            res = res.substring(0,lastPoint) + res.substring(lastPoint+1,res.length());
            firstPoint = res.indexOf(".");
            lastPoint = res.lastIndexOf(".");
        }

        if (res.length()==0) {
            this.lastCursorPos = -1;
            this.fireValidationFailed("must be a numeric value");
            return;
        }
//        if (firstPoint == -1) {
//            this.fireValidationFailed("decimal point is missing");
//        }

        try {
            this.lastAsText = res;
            this.setFieldValue(Double.parseDouble(res));


        } catch (NumberFormatException nfe) {
            this.lastCursorPos = -1;
            this.fireValidationFailed("must be a numeric value");
            return;
        }


    }

    @Override
    public void insertUpdate(DocumentEvent e) {
        this.lastCursorPos = inputTextField.getCaretPosition()+e.getLength();
        this.changedUpdate(e);
    }

    @Override
    public void removeUpdate(DocumentEvent e) {
        if (e.getOffset() < inputTextField.getCaretPosition()) {
            lastCursorPos = inputTextField.getCaretPosition()-(e.getLength());
        } else {
            lastCursorPos = inputTextField.getCaretPosition();
        }
        this.changedUpdate(e);
    }



}
