/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool.parameters.gui;

import net.aavalla.utils.jobpool.parameters.FieldInstance;
import java.awt.GridLayout;
import java.util.HashMap;
import javax.swing.BorderFactory;
import javax.swing.JRadioButton;

/**
 *
 * @author Lari Natri
 */
public class EnumComponent<E extends Enum<E>> extends ValueComponent<E> {

    private Class<E> enumType;
    private final HashMap<E,JRadioButton> enumToButtonMap;

    /** Creates new form EnumComponent */
    @SuppressWarnings("unchecked")
    public EnumComponent(FieldInstance field, boolean allowEdit) {
        super(field, allowEdit);
        initComponents();

        this.enumToButtonMap = new HashMap<E,JRadioButton>(8);
        
        this.enumType = (Class<E>)this.getValueType();



        for(E ec : enumType.getEnumConstants()) {
            JRadioButton rb = new JRadioButton(ec.toString());
            rb.setActionCommand(ec.name());
            rb.setToolTipText(this.getToolTipText());
            this.add(rb);
            this.enumButtonGroup.add(rb);
            this.enumToButtonMap.put(ec,rb);
            
            if (allowEdit) {
                rb.addActionListener(new java.awt.event.ActionListener() {
                    @Override
                    public void actionPerformed(java.awt.event.ActionEvent evt) {
                        String s = enumButtonGroup.getSelection().getActionCommand();
                        for (E ec : enumType.getEnumConstants()) {
                            if (s.equalsIgnoreCase(ec.name())) {
                                EnumComponent.this.setFieldValue(ec);
                                break;
                            }
                        }
                    }
                });
                rb.setEnabled(true);
            } else {
                rb.setEnabled(false);
            }
        }

        this.setFieldValue((E)field.getValue()); // set default value

        this.setBorder(BorderFactory.createEtchedBorder());
        this.setLayout(new GridLayout(enumButtonGroup.getButtonCount(),1));

        this.validate();

    }

    @Override
    public void dispose() {
        this.enumToButtonMap.clear();
        super.dispose();
    }



    @Override
    protected void updateUIWithValue(E newValue) {
        this.enumButtonGroup.setSelected(this.enumToButtonMap.get(newValue).getModel(),true);
    }


    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        enumButtonGroup = new javax.swing.ButtonGroup();

        setName("Form"); // NOI18N
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup enumButtonGroup;
    // End of variables declaration//GEN-END:variables

}
