/*
 * Copyright 2010 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package net.aavalla.utils.jobpool.parameters.gui;

import net.aavalla.utils.jobpool.parameters.FieldInstance;
import net.aavalla.utils.jobpool.parameters.SyncEnumSet;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JLabel;

/**
 *
 * @author Lari Natri
 */
public class EnumSetComponent<E extends Enum<E>> 
        extends ValueComponent<SyncEnumSet<E>>
        implements ActionListener
{

    private final Class<E> enumClass;
    private final HashMap<E,JCheckBox> enumToBox = new HashMap<E,JCheckBox>(4);
    private final HashMap<JCheckBox,E> boxToEnum = new HashMap<JCheckBox,E>(4);

    @SuppressWarnings("unchecked")
    public EnumSetComponent(FieldInstance field, boolean allowEdit) {
        super(field, allowEdit);
        
        this.enumClass = ((SyncEnumSet<E>)field.getValue()).getElementType();

        for (E e : SyncEnumSet.allOf(enumClass)) { // loop through possible values
            JLabel l = new JLabel(e.toString());
            JCheckBox b = new JCheckBox();
            b.setEnabled(allowEdit);
            b.setToolTipText(this.getToolTipText());
            l.setToolTipText(this.getToolTipText());
            l.setLabelFor(b);
            if (allowEdit) {
                b.addActionListener(this);
            }
            this.add(l);
            this.add(b);
            
            this.enumToBox.put(e, b);
            this.boxToEnum.put(b, e);
        }
        

        this.setFieldValue((SyncEnumSet<E>)field.getValue()); // set default value

        this.setBorder(BorderFactory.createEtchedBorder());
        this.setLayout(new GridLayout(boxToEnum.size(),2));

//        this.setEnabled(allowEdit);
        this.setVisible(true);
        this.validate();
    }

    @Override
    protected void updateUIWithValue(SyncEnumSet<E> newValue) {

        for (JCheckBox b : this.boxToEnum.keySet()) {
            b.setSelected(false);
        }

        for (E e : newValue) {
            this.enumToBox.get(e).setSelected(true);
        }
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        SyncEnumSet<E> newSet = SyncEnumSet.noneOf(enumClass);
        for (JCheckBox b : this.boxToEnum.keySet()) {
            if (b.isSelected()) {
                newSet.add(this.boxToEnum.get(b));
            }
        }
        this.setFieldValue(newSet);
    }



}
