/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool.parameters.gui;

import net.aavalla.utils.jobpool.parameters.FieldInstance;
import net.aavalla.utils.jobpool.parameters.OutputFile;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

/**
 *
 * @author Lari Natri
 */
public class FileChooserComponent extends ValueComponent<File> {

    private final JFileChooser jFC;

    /** Creates new form FileChooser */
    public FileChooserComponent(final FieldInstance field, boolean allowEdit) {
        super(field, allowEdit);
        initComponents();

        this.jFC = new JFileChooser();
        jFC.setFileSelectionMode(JFileChooser.FILES_ONLY);

        if (FileChooserComponent.this.getValueType().equals(OutputFile.class)) {
            fileChooserButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/aavalla/utils/jobpool/parameters/gui/resources/document-save-as.png"))); //NOI18N
        } else {
            fileChooserButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/aavalla/utils/jobpool/parameters/gui/resources/document-open.png"))); //NOI18N
        }

        this.fileChooserButton.setVisible(allowEdit);
        this.fileChooserButton.setToolTipText(this.getToolTipText());
        this.fileNameText.setToolTipText("Selected file name is displayed here");



        if (allowEdit) {
            fileChooserButton.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    int retVal;
                    if (field.getType().equals(OutputFile.class)) {
                        retVal = jFC.showSaveDialog(FileChooserComponent.this);
                    } else {
                        retVal = jFC.showOpenDialog(FileChooserComponent.this);
                    }

                    if (retVal == JFileChooser.APPROVE_OPTION) {
                        File file = jFC.getSelectedFile();
                        boolean fileOK = true;
                        if (FileChooserComponent.this.getValueType().equals(OutputFile.class)) {
                            if (file.exists()) {
                                fileOK = JOptionPane.showConfirmDialog(FileChooserComponent.this,
                                         "Do you really want to overwrite "+file.getName()+"?",
                                         "Overwrite file?", JOptionPane.YES_NO_OPTION) ==
                                         JOptionPane.YES_OPTION;
                            }
                        }
                        if (fileOK) {
                            FileChooserComponent.this.setFieldValue(file);
                        }
                    }
                }
            });
        }

        this.setFieldValue((File)field.getValue()); // set default value
    }

    @Override
    protected void updateUIWithValue(File newValue) {
        fileNameText.setText(newValue==null ? "" : newValue.toString());
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        fileChooserButton = new javax.swing.JButton();
        fileNameText = new javax.swing.JTextField();

        setName("Form"); // NOI18N

        fileChooserButton.setText("Browse"); // NOI18N
        fileChooserButton.setName("fileChooserButton"); // NOI18N

        fileNameText.setEditable(false);
        fileNameText.setName("fileNameText"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(fileChooserButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(fileNameText, javax.swing.GroupLayout.DEFAULT_SIZE, 229, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                .addComponent(fileChooserButton)
                .addComponent(fileNameText, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton fileChooserButton;
    private javax.swing.JTextField fileNameText;
    // End of variables declaration//GEN-END:variables


}
