/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool.parameters.gui;

import net.aavalla.utils.gui.fontchooser.FontChooserDialog;
import net.aavalla.utils.jobpool.parameters.FieldInstance;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 *
 * @author Lari Natri
 */
public class FontChooserComponent extends ValueComponent<Font> {

    /** Creates new form FileChooser */
    public FontChooserComponent(final FieldInstance field, boolean allowEdit) {
        super(field, allowEdit);
        initComponents();

        this.fontChooserButton.setVisible(allowEdit);
        this.fontChooserButton.setToolTipText(this.getToolTipText());
        this.fontChooserButton.setIcon(new javax.swing.ImageIcon(getClass().getResource("/net/aavalla/utils/jobpool/parameters/gui/resources/preferences-desktop-font.png"))); //NOI18N
        this.fontText.setToolTipText("Selected font is displayed here");

        if (field.getValue()==null) {
            this.setFieldValue(this.getFont());
        } else {
            this.setFieldValue((Font)field.getValue());
        }

        if (allowEdit) {
            this.fontChooserButton.addActionListener(new ActionListener() {
                @Override
                public void actionPerformed(ActionEvent e) {
                    Font f = null;
                    if (field.getValue()!=null) {
                        f = (Font)field.getValue();
                    }

                    FontChooserDialog fd = new FontChooserDialog(null, 
                            "Select font", true, f, false);
                    fd.setVisible(true);
                    if (!fd.wasCancelled()) {
                        FontChooserComponent.this.setFieldValue(fd.getChosenFont());
                    }
                }
            });
        }

    }

    @Override
    protected void updateUIWithValue(Font newValue) {
        if (newValue==null) {
            this.fontText.setFont(this.getFont());
            this.fontText.setText("No font selected");
            return;
        }
        this.fontText.setFont(newValue);
        StringBuilder sb = new StringBuilder(newValue.getFamily());
        if (newValue.isBold() && newValue.isItalic()) {
            sb.append(", Bold + Italic");
        } else if (newValue.isBold()) {
            sb.append(", Bold");
        } else if (newValue.isItalic()) {
            sb.append(", Italic");
        } else {
            sb.append(", Plain");
        }
        this.fontText.setText(sb.toString());
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        fontChooserButton = new javax.swing.JButton();
        fontText = new javax.swing.JTextField();

        setName("Form"); // NOI18N

        fontChooserButton.setText("Choose"); // NOI18N
        fontChooserButton.setName("fontChooserButton"); // NOI18N

        fontText.setEditable(false);
        fontText.setName("fontText"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(fontChooserButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(fontText, javax.swing.GroupLayout.DEFAULT_SIZE, 229, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                .addComponent(fontChooserButton)
                .addComponent(fontText, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton fontChooserButton;
    private javax.swing.JTextField fontText;
    // End of variables declaration//GEN-END:variables


}
