/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool.parameters.gui;

import net.aavalla.utils.jobpool.parameters.FieldInstance;
import java.awt.Color;
import java.text.ParseException;
import javax.swing.JFormattedTextField;
import javax.swing.JTextArea;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.JTextComponent;

/**
 *
 * @author Lari Natri
 */
public class StringComponent extends ValueComponent<String> implements DocumentListener {

    private final JTextComponent inputTextField;
    private int lastCursorPos = -1;
    
    public StringComponent(FieldInstance field, boolean allowEdit) {
        super(field, allowEdit);
        int lines = field.getLines();
        if (lines>1) {
            this.inputTextField = new JTextArea();
            ((JTextArea)this.inputTextField).setRows(field.getLines());

        } else {
            this.inputTextField = new JFormattedTextField();
            ((JFormattedTextField)this.inputTextField).setColumns(20);
        }


        this.inputTextField.getDocument().addDocumentListener(this);
        this.inputTextField.setToolTipText(this.getToolTipText());

        this.setFieldValue((String)field.getValue()); // set default value

        this.inputTextField.setBackground(Color.WHITE);
        this.inputTextField.setEditable(allowEdit);
//        this.setEnabled(allowEdit);

        this.add(this.inputTextField);


        this.validate();

    }

    @Override
    protected void updateUIWithValue(String newValue) {
        inputTextField.getDocument().removeDocumentListener(this);
        inputTextField.setText(newValue.toString());

        if (lastCursorPos >= 0) {
            if (lastCursorPos >= newValue.toString().length()) {
                inputTextField.setCaretPosition(newValue.toString().length());
            } else {
                inputTextField.setCaretPosition(lastCursorPos);
            }
        }
        inputTextField.getDocument().addDocumentListener(this);
    }

    @Override
    public void changedUpdate(DocumentEvent e) {
        if (inputTextField instanceof JFormattedTextField) {
            try {
                ((JFormattedTextField)inputTextField).commitEdit();
            } catch (ParseException ex) {}
        }
        this.setFieldValue(inputTextField.getText());
    }

    @Override
    public void insertUpdate(DocumentEvent e) {
        lastCursorPos = inputTextField.getCaretPosition()+e.getLength();
        this.changedUpdate(e);
    }

    @Override
    public void removeUpdate(DocumentEvent e) {
        if (e.getOffset() < inputTextField.getCaretPosition()) {
            lastCursorPos = inputTextField.getCaretPosition()-(e.getLength());
        } else {
            lastCursorPos = inputTextField.getCaretPosition();
        }
        this.changedUpdate(e);
    }



}
