/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool.parameters.gui;

import net.aavalla.utils.general.Disposeable;
import net.aavalla.utils.jobpool.parameters.FieldInstance;
import net.aavalla.utils.jobpool.parameters.FieldListener;
import java.awt.GridLayout;
import javax.print.attribute.UnmodifiableSetException;

/**
 *
 * @author Lari Natri
 */
public abstract class ValueComponent<T> extends javax.swing.JPanel implements FieldListener<T>, Disposeable {

    private final FieldInstance field;


    /** Creates new form ValueComponent */
    public ValueComponent(FieldInstance field, boolean allowEdit) {
        initComponents();

        this.setLayout(new GridLayout(1,0));

        this.setToolTipText(field.getFullHtmlDescription());

        this.field = field;
        this.field.addFieldListener(this,true);

//        this.setEnabled(allowEdit); // should be done in derived classes also
        
    }

    @Override
    public void dispose() {
        this.field.removeFieldListener(this);
    }

    @Override
    public final void fieldValueChanged(FieldInstance field, T newValue) {
        this.updateUIWithValue(newValue);
    }

    @Override
    public final void validationFailed(FieldInstance field, String reason) {
        // this is handled in ParameterDialog
    }

    protected final Class getValueType() {
        return this.field.getType();
    }

    /**
     * Fires fireValidationFailed() in the underlying FieldInstance. This should
     * be called from deriving classes, if they use internal validation.
     * @param reason
     */
    protected final void fireValidationFailed(final String reason) {
        this.field.fireValidationFailed(reason);
    }

    /**
     * Tries to set a given value to the underlying FieldInstance. Should be
     * called from derived classes.
     * @param value new value to set
     */
    protected final void setFieldValue(final T value) {
        try {
            this.field.setValue(value);
        } catch (UnmodifiableSetException ex) {
        }
        
    }

    /**
     * Deriving classes should have their UI updated within this method.
     * @param newValue New legal value from the underlying FieldInstance
     */
    protected abstract void updateUIWithValue(T newValue);


    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        setName("Form"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 243, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 22, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents


    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables

}
