/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool.parameters.validators;

import net.aavalla.utils.jobpool.parameters.*;

/**
 *
 * @author Lari Natri
 */
    public class IntegerMinMaxValidator extends ParameterValidator<Integer> {
        
        private Integer min;
        private Integer max;
        private String uiRules;

        public IntegerMinMaxValidator(FieldInstance field, String[] settings) {
            super(field,settings);

            if (settings[0]==null || settings[0].equals(FieldAnnotations.NULL)) {
                min = null;
            } else {
                min = Integer.parseInt(settings[0].replaceAll("[^0123456789-]", ""));
            }
            if (settings[1]==null || settings[1].equals(FieldAnnotations.NULL)) {
                max = null;
            } else {
                max = Integer.parseInt(settings[1].replaceAll("[^0123456789-]", ""));
            }

            StringBuilder uiRulesBuilder = new StringBuilder("");
            if (min != null) {
                uiRulesBuilder.append("minimum: ");
                uiRulesBuilder.append(min);
            }
            if (max != null) {
                if (uiRulesBuilder.length()>0) uiRulesBuilder.append(", ");
                uiRulesBuilder.append("maximum: ");
                uiRulesBuilder.append(max);
            }
            uiRules = uiRulesBuilder.toString();
        }

        @Override
        public String validate(Integer param) {
            if (min != null && param < min) {
                return "Minimum allowed value is "+min;
            }
            if (max !=null && param > max) {
                return "Maximum allowed value is "+max;
            }
            return null;

        }

        @Override
        public String uiValidationRules() {
            return uiRules;
        }

    }
