/*
 * Copyright 2009 Lari Natri <lari.natri@gmail.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


package net.aavalla.utils.jobpool.parameters.validators;

import net.aavalla.utils.jobpool.parameters.*;

/**
 *
 * @author Lari Natri
 * @param <T> The type of objects this validator is for
 */
public abstract class ParameterValidator<T> {
    
    protected final FieldInstance field;

    public ParameterValidator(FieldInstance field, String[] settings) {

        this.field = field;

        if (settings==null) {
            settings = new String[0];
            return;
        }

        for (String s : settings) {
            if (s == null || s.equals(FieldAnnotations.NULL)) {
                s = null;
            }
        }

    }


    /**
     * Validates a parameter of type T.
     * @param param Parameter to validate
     * @param parameters Parameters to pass to the validator (for reusing of validator classes)
     * @return null if parameter is valid, a UI String otherwise
     */
    public abstract String validate(final T param);

    /**
     * Returns the rules of this validator. To be displayed in the user
     * interface.
     */
    public abstract String uiValidationRules();

}
